      subroutine bcond(idx)
C **********************************************************************
C *                                                                    *
C * FUNCTION    :  Applies open boundary conditions.                   *
C *                                                                    *
C *                Closed boundary conditions are automatically        *
C *                enabled through specification of the masks, dum,    *
C *                dvm and fsm, in which case the open boundary        *
C *                conditions, included below, will be overwritten.    *
C *                                                                    *
C *                                The C-Grid                          *
C *                                **********                          *
C *                                                                    *
C *                The diagram below and some of the text was provided *
C *                by D.-S. Ko. It is for the case where u and v are   *
C *                the primary boundary conditions together with t and *
C *                s (co-located with e). e = el itself is rather      *
C *                unimportant and is substituted from an adjacent     *
C *                interior point.                                     *
C *                                                                    *
C *                The dotted line (.......) bounds the interior       *
C *                (non-boundary) grid points. In general, only those  *
C *                variables in the interior are computed and          *
C *                variables at open boundary have to be specified.    *
C *                All interpolations are centered in space except     *
C *                those at lateral open boundary where an upstream    *
C *                scheme is usually used. "BU" indictates a line of   *
C *                points where the boundary conditions should be      *
C *                specified.                                          *
C *                                                                    *
C *                Horizontal locations of e(el), t and s (etc.) are   *
C *                coincident. Unused points are indicated by "NU" and *
C *                "*". However, for attractive output, adjacent       *
C *                interior values may be filled in at these points.   *
C *                                                                    *
C *                People not acquainted with sigma coordinates have   *
C *                often asked what kind of boundary condition is      *
C *                applied along closed horizontal boundaries.         *
C *                Although the issue is not as important as it might  *
C *                be  for z-level grids, a direct answer is "half-    *
C *                slip" which, of course, is between free slip and    *
C *                non-slip.                                           *
C *                                                                    **********
C-------------------------------------------------------------------------------*
C     |                               N O R T H                                 *
C     |                                                                         *
C     |    1     2     3           i-1   i    i+1         im-2  im-1   im       *
C-----+-------------------------------------------------------------------------*
C     |   NU BC BC                                                    BC BC     *
C     |    v  v  v                                                     v  v     *
C     |                                                                         *
C     |BC> u* e  u  e  u  e  .  .  u  e  u  e  u  e  .  .  u  e  u  e  u  e  <BC*
C     |    |     |     |           |     |     |           |     |     |        *
C  jm |BC> +--v--+--v--+--v--   .  +--v--+--v--+--v--   .  +--v--+--v--+--v- <BC*
C     |    |     | ....|...........|.....|.....|...........|.....|.... |        *
C     |    u* e  u :e  u  e  .  .  u  e  u  e  u  e  .  .  u  e  u  e: u  e     *
C     |    |     | :   |           |     |     |           |     |   : |        *
C jm-1|    +--v--+--v--+--v--   .  +--v--+--v--+--v--   .  +--v--+--v--+--v-    *
C     |    |     | :   |           |     |     |           |     |   : |        *
C     |    u* e  u :e  u  e  .  .  u  e  u  e  u  e  .  .  u  e  u  e: u  e     *
C     |    |     | :   |           |     |     |           |     |   : |        *
C jm-2|    +--v--+--v--+--v--   .  +--v--+--v--+--v--   .  +--v--+--v--+--v-    *
C     |            :                                                 :          *
C W   |       .  . :.  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .: .  .    E*
C E   |            :                    INTERIOR                     :         A*
C S   |       .    :.     .     .     .     .     .     .     .     .:    .    S*
C T   |            :                                                 :         T*
C     |    |     | :   |           |     |     |           |     |   : |        *
C     |    u* e  u :e  u  e  .  .  u  e  u  e  u  e  .  .  u  e  u  e: u  e     *
C     |    |     | :   |           |     |     |           |     |   : |        *
C   3 |    +--v--+--v--+--v--   .  +--v--+--v--+--v--   .  +--v--+--v--+--v-    *
C     |    |     | :   |           |     |     |           |     |   : |        *
C     |    u* e  u :e  u  e  .  .  u  e  u  e  u  e  .  .  u  e  u  e: u  e     *
C     |    |     | ....|...........|.....|.....|...........|.....|.... |        *
C   2 |BC> +--v--+--v--+--v--   .  +--v--+--v--+--v--   .  +--v--+--v--+--v- <BC*
C     |    |     |     |           |     |     |           |     |     |        *
C     |BC> u* e  u  e  u  e  .  .  u  e  u  e  u  e  .  .  u  e  u  e  u  e  <BC*
C     |    |     |     |           |     |     |           |     |     |      --*
C   1 |NU> +--v*-+--v*-+--v*-   .  +--v*-+--v*-+--v*-   .  +--v*-+--v*-+--v* <NU*
C     |                                                                         *
C     |    ^  ^  ^                                                     ^  ^     *
C     |   NU BC BC                                                    BC BC     *
C-----+-------------------------------------------------------------------------*
C     |    1     2     3           i-1   i     i+1         im-2  im-1  im       *
C     |                                                                         *
C     |                                S O U T H                                *
C-------------------------------------------------------------------------------*
C                                                                               *
C *******************************************************************************
C
      implicit none
C
      include 'pomgl2007.h'
C
      integer nbcmax
      parameter (nbcmax=5000)
      integer idx,npos,nt,ifirst,nrmax,n1,nrec
      real ga,u1,wm,wgt,t1,uf1,uf2,vf1,vf2,dtbc
      real hbc(nbcmax),hbc1(nbcmax),hbc2(nbcmax)
      real ubc(nbcmax),ubc1(nbcmax),ubc2(nbcmax)
      real vbc(nbcmax),vbc1(nbcmax),vbc2(nbcmax)
      real u3bc(nbcmax,kb),u3bc1(nbcmax,kb),u3bc2(nbcmax,kb)
      real v3bc(nbcmax,kb),v3bc1(nbcmax,kb),v3bc2(nbcmax,kb)
      real t3bc(nbcmax,kb),t3bc1(nbcmax,kb),t3bc2(nbcmax,kb)
      
      integer i,j,k,ktype,nr
C
      save 
      
      data ifirst/0/
      nibc=iparm(46)
      
C-----------------------------------------------------------------------
C ON FIRST TIME STEP OPEN REQUIRED INPUT DATA FILES
C AND SET PARAMETERS (OPEN BOUNDARY CASE ONLY)
C
      IF(IFIRST.EQ.0.AND.NIBC.GT.0) THEN
C
C DTWIND (HOURS) IS AN INPUT PARAMETER READ FROM INPUT FILE  
C AT START OF MAIN PROGRAM AND IS IN COMMON BLOCK 
C 
C use DTBC (HOURS) instead to reflect the pomgl output
       DTBC=1.
       print *, '+++++++++++++++++++++++++++'
       print *, 'DTWIND=',DTWIND
       print *, 'DTBC=',dtbc
       print *, '+++++++++++++++++++++++++++'
C 
c       NRMAX=IHRS/IFIX(DTWIND)+1
       NRMAX=IHRS/IFIX(DTBC)+1
       PRINT *,NRMAX

       K=INDEX(FINPUT,' ') 
       OPEN(18,FILE=FINPUT(1:K-1)//'.bc3 ',FORM='FORMATTED',
     1  STATUS='OLD')

c skip appropriate number of records if required

       IF(NRSKIP.GT.0) THEN
        DO NR=1,NRSKIP
         DO N=1,NIBC
          READ(18,*) I,J,HBC1(N),UBC1(N),VBC1(N)
          READ(18,*)(U3BC1(N,K),K=1,KB)
          READ(18,*)(V3BC1(N,K),K=1,KB)
          READ(18,*)(T3BC1(N,K),K=1,KB)
         ENDDO
        ENDDO
       ENDIF
       
       NREC=0

       N1=0
       IFIRST=1

      ENDIF
     
C END OF SETTING PARAMETERS FOR FIRST TIME STEP  
C-----------------------------------------------------------------

      IF(NIBC.GT.0) THEN
C-----------------------------------------------------------------------
C  OPEN BOUNDARY CASE
C-----------------------------------------------------------------------
	
C DETERMINE RECORD NUMBER AT BEGINNING OF INTERVAL

c      N=IFIX(TIME/DTWIND)+1
      N=IFIX(TIME/DTBC)+1
c      T1=(N-1)*DTWIND
      T1=(N-1)*DTBC

C CHECK IF NEW FIELDS ARE REQUIRED

      IF(N.NE.N1) THEN
       N1=N

C FIRST TIME THROUGH GET VELEOCITIES AT BEGINNING OF INTERVAL

       IF(N1.EQ.1.AND.NREAD.NE.1) THEN

        NREC=NREC+1

C        NPOS=(NREC-1)*(NIBC+1)*8
C        CALL XDRSETPOS(18,NPOS)
C        CALL XDRARRAY_FLOAT(18,UBC1,NIBC)
C        CALL XDRARRAY_FLOAT(18,VBC1,NIBC)

        DO N=1,NIBC
         READ(18,*) I,J,HBC1(N),UBC1(N),VBC1(N)
c         IF(MODE.NE.2) THEN
          READ(18,*)(U3BC1(N,K),K=1,KB)
          READ(18,*)(V3BC1(N,K),K=1,KB)
          READ(18,*)(T3BC1(N,K),K=1,KB)
c         ENDIF
        ENDDO

       ELSE

C  INCREMENT NREC FOR RESTART

        IF(N1.EQ.1) THEN
         NREC=NREC+1
        ELSE

C  OTHERWISE GET OLD FIELD FROM END OF INTERVAL

         DO I=1,NIBC
          HBC1(I)=HBC2(I)
          UBC1(I)=UBC2(I)
          VBC1(I)=VBC2(I)
          DO K=1,KB
           U3BC1(I,K)=U3BC2(I,K)          
           V3BC1(I,K)=V3BC2(I,K)          
           T3BC1(I,K)=T3BC2(I,K)          
          ENDDO
         ENDDO
        ENDIF
       ENDIF

C  READ FIELDS AT END OF INTERVAL

       NREC=NREC+1
       IF(NREC.LE.NRMAX) THEN
        DO N=1,NIBC
         READ(18,*) I,J,HBC2(N),UBC2(N),VBC2(N)
          READ(18,*)(U3BC2(N,K),K=1,KB)
          READ(18,*)(V3BC2(N,K),K=1,KB)
          READ(18,*)(T3BC2(N,K),K=1,KB)
        ENDDO

        NT=MIN(N1+1,NRMAX-1)

       ENDIF
      ENDIF

C LINEARLY INTERPOLATEVELOCITY COMPONENTS IN TIME


c      WGT=(TIME-T1)/DTWIND
      WGT=(TIME-T1)/DTBC
      DO I=1,NIBC
       HBC(I)=(1.-WGT)*HBC1(I)+WGT*HBC2(I)
       UBC(I)=(1.-WGT)*UBC1(I)+WGT*UBC2(I)
       VBC(I)=(1.-WGT)*VBC1(I)+WGT*VBC2(I)
       DO K=1,KB
        U3BC(I,K)=(1.-WGT)*U3BC1(I,K)+WGT*U3BC2(I,K)
        V3BC(I,K)=(1.-WGT)*V3BC1(I,K)+WGT*V3BC2(I,K)
        T3BC(I,K)=(1.-WGT)*T3BC1(I,K)+WGT*T3BC2(I,K)-TBIAS
       ENDDO
      ENDDO
      
C-----------------------------------------------------------------------
c
c   ktype and location of adjacent interior grid points for pomgl nested grid
c
c    1 - east
c    2 - north and east
c    3 - north
c    4 - north and west
c    5 - west
c    6 - south and west  
c    7 - south
c    8 - south and east
c    0 - cosmetic boundary condition (not used in calculations)
c
c               +---+
c               | 7 |
c           +---+---+---+
c           | 8 |   | 6 |
c       +---+---+---+---+---+
c       | 1 |   |   |   | 5 |
c       +---+---+---+---+---+
c           | 2 |   | 4 |
c           +---+---+---+
c               | 3 |
c               +---+
c
C-----------------------------------------------------------------------
C
C     External (2-D) boundary conditions:
C
      if(idx.eq.1) then
   
C   elevation (set boundary values to nearest interior points)

      if(nibc.gt.0) then
        do n=1,nibc
         i=ibc(1,n)
         j=ibc(2,n)
         ktype=ibc(3,n)
         elf(i,j)=hbc(n)
         if(ktype.eq.1) then
          elf(i,j)=elf(i+1,j)
         endif
         if(ktype.eq.2) then
          elf(i,j)=0.5*(elf(i+1,j)+elf(i,j+1))
         endif
         if(ktype.eq.3) then
          elf(i,j)=elf(i,j+1)
         endif
         if(ktype.eq.4) then
          elf(i,j)=0.5*(elf(i-1,j)+elf(i,j+1))
         endif
         if(ktype.eq.5) then
          elf(i,j)=elf(i-1,j)
         endif
         if(ktype.eq.6) then
          elf(i,j)=0.5*(elf(i-1,j)+elf(i,j-1))
         endif
         if(ktype.eq.7) then
          elf(i,j)=elf(i,j-1)
         endif
         if(ktype.eq.8) then
          elf(i,j)=0.5*(elf(i+1,j)+elf(i,j-1))
         endif
        end do
       endif
        
        do j=1,jm
          do i=1,im
            elf(i,j)=elf(i,j)*fsm(i,j)
          end do
        end do

c  adjustment of water level (for nested grids)

        wlavg=0
        do j=1,jm
          do i=1,im
            if(fsm(i,j).ne.0.) wlavg=wlavg+elf(i,j)
          end do
        end do
        wlavg=wlavg/nfsm
        do j=1,jm
          do i=1,im
            if(fsm(i,j).ne.0.) elf(i,j)=elf(i,j)-wlavg
          end do
        end do
        wltot=wltot+wlavg
        
        return
C
      else if(idx.eq.2) then

C-----------------------------------------------------------------------
C
C     External (2-D) velocity:
C       (substitution)
C
       if(nibc.gt.0) then
        do n=1,nibc
         i=ibc(1,n)
         j=ibc(2,n)
         ktype=ibc(3,n)
         if(ktype.eq.0) then
          uaf(i,j)=ubc(n)
          vaf(i,j)=vbc(n)
         endif     
         if(ktype.eq.1) then
          uaf(i+1,j)=ubc(n)
          vaf(i,j)=vbc(n)
         endif
         if(ktype.eq.2) then
          uaf(i+1,j)=ubc(n)
          vaf(i,j+1)=vbc(n)
         endif
         if(ktype.eq.3) then
          uaf(i,j)=ubc(n)
          vaf(i,j+1)=vbc(n)
         endif
         if(ktype.eq.4) then
          uaf(i,j)=ubc(n)
          vaf(i,j+1)=vbc(n)
         endif
         if(ktype.eq.5) then
          uaf(i,j)=ubc(n)
          vaf(i,j)=vbc(n)
         endif
         if(ktype.eq.6) then
          uaf(i,j)=ubc(n)
          vaf(i,j)=vbc(n)
         endif
         if(ktype.eq.7) then
          uaf(i,j)=ubc(n)
          vaf(i,j)=vbc(n)
         endif
         if(ktype.eq.8) then
          uaf(i+1,j)=ubc(n)
          vaf(i,j)=vbc(n)
         endif

        end do
       endif
       
        do j=1,jm
          do i=1,im
            uaf(i,j)=uaf(i,j)*dum(i,j)
            vaf(i,j)=vaf(i,j)*dvm(i,j)
          end do
        end do
C
        return
C
      else if(idx.eq.3) then
C
C-----------------------------------------------------------------------
C
C     Internal (3-D) boundary conditions:
C
C     Velocity (radiation conditions; smoothing is used in the direction
C     tangential to the boundaries):
C
       if(nibc.gt.0) then
        do n=1,nibc
         i=ibc(1,n)
         j=ibc(2,n)
         ktype=ibc(3,n)
         ga=sqrt(h(i,j)/hmax)
         do k=1,kbm1
          if(ktype.eq.0) then
           uf(i,j,k)=u3bc(n,k)
           vf(i,j,k)=v3bc(n,k)
          endif
          if(ktype.eq.1) then
           if(u(i+1,j,k).ge.0.) then
            uf(i+1,j,k)=u3bc(n,k)
           else
            uf(i+1,j,k)=ga*(.25e0*u(i+2,j-1,k)+.5e0*u(i+2,j,k)
     $                    +.25e0*u(i+2,j+1,k))
     $                 +(1.e0-ga)*(.25e0*u(i+1,j-1,k)+.5e0*u(i+1,j,k)
     $                   +.25e0*u(i+1,j+1,k))
           endif
           uf(i,j,k)=uf(i+1,j,k)
           vf(i,j,k)=v3bc(n,k)
          endif

          if(ktype.eq.2) then
           if(u(i+1,j,k).ge.0.) then
            uf(i+1,j,k)=u3bc(n,k)
           else
            uf(i+1,j,k)=ga*(.25e0*u(i+2,j-1,k)+.5e0*u(i+2,j,k)
     $                    +.25e0*u(i+2,j+1,k))
     $                 +(1.e0-ga)*(.25e0*u(i+1,j-1,k)+.5e0*u(i+1,j,k)
     $                   +.25e0*u(i+1,j+1,k))
           endif
           uf(i,j,k)=uf(i+1,j,k)
           if(v(i,j+1,k).le.0.) then
            vf(i,j+1,k)=v3bc(n,k)
           else
            vf(i,j+1,k)=ga*(.25e0*v(i-1,j+2,k)+.5e0*v(i,j+2,k)
     $                    +.25e0*v(i+1,j+2,k))
     $                 +(1.e0-ga)*(.25e0*v(i-1,j+1,k)+.5e0*v(i,j+1,k)
     $                   +.25e0*v(i+1,j+1,k))
           endif
           vf(i,j,k)=vf(i,j+1,k)
          endif

          if(ktype.eq.3) then
           if(v(i+1,j,k).le.0.) then
            vf(i,j+1,k)=v3bc(n,k)
           else
            vf(i,j+1,k)=ga*(.25e0*v(i-1,j+2,k)+.5e0*v(i,j+2,k)
     $                    +.25e0*v(i+1,j+2,k))
     $                 +(1.e0-ga)*(.25e0*v(i-1,j+1,k)+.5e0*v(i,j+1,k)
     $                   +.25e0*v(i+1,j+1,k))
           endif
           vf(i,j,k)=vf(i,j+1,k)
           uf(i,j,k)=u3bc(n,k)
          endif

          if(ktype.eq.4) then
           if(u(i,j,k).le.0.) then
            uf(i,j,k)=u3bc(n,k)
           else
            uf(i,j,k)=ga*(.25e0*u(i-1,j-1,k)+.5e0*u(i-1,j,k)
     $                     +.25e0*u(i-1,j+1,k))
     $                  +(1.e0-ga)*(.25e0*u(i,j-1,k)+.5e0*u(i,j,k)
     $                    +.25e0*u(i,j+1,k))
           endif
           if(v(i+1,j,k).le.0.) then
            vf(i,j+1,k)=v3bc(n,k)
           else
            vf(i,j+1,k)=ga*(.25e0*v(i-1,j+2,k)+.5e0*v(i,j+2,k)
     $                    +.25e0*v(i+1,j+2,k))
     $                 +(1.e0-ga)*(.25e0*v(i-1,j+1,k)+.5e0*v(i,j+1,k)
     $                   +.25e0*v(i+1,j+1,k))
           endif
           vf(i,j,k)=vf(i,j+1,k)
          endif

          if(ktype.eq.5) then
           if(u(i,j,k).le.0.) then
            uf(i,j,k)=u3bc(n,k)
           else
            uf(i,j,k)=ga*(.25e0*u(i-1,j-1,k)+.5e0*u(i-1,j,k)
     $                     +.25e0*u(i-1,j+1,k))
     $                  +(1.e0-ga)*(.25e0*u(i,j-1,k)+.5e0*u(i,j,k)
     $                    +.25e0*u(i,j+1,k))
           endif
           vf(i,j,k)=v3bc(n,k)
          endif

          if(ktype.eq.6) then
           if(u(i,j,k).le.0.) then
            uf(i,j,k)=u3bc(n,k)
           else
            uf(i,j,k)=ga*(.25e0*u(i-1,j-1,k)+.5e0*u(i-1,j,k)
     $                     +.25e0*u(i-1,j+1,k))
     $                  +(1.e0-ga)*(.25e0*u(i,j-1,k)+.5e0*u(i,j,k)
     $                    +.25e0*u(i,j+1,k))
           endif
           if(v(i,j,k).le.0.) then
            vf(i,j,k)=v3bc(n,k)
           else
            vf(i,j,k)=ga*(.25e0*v(i-1,j-1,k)+.5e0*v(i,j-1,k)
     $                     +.25e0*v(i+1,j-1,k))
     $                  +(1.e0-ga)*(.25e0*v(i-1,j,k)+.5e0*v(i,j,k)
     $                    +.25e0*v(i+1,j,k))
           endif
          endif

          if(ktype.eq.7) then
           if(v(i,j,k).le.0.) then
            vf(i,j,k)=v3bc(n,k)
           else
            vf(i,j,k)=ga*(.25e0*v(i-1,j-1,k)+.5e0*v(i,j-1,k)
     $                     +.25e0*v(i+1,j-1,k))
     $                  +(1.e0-ga)*(.25e0*v(i-1,j,k)+.5e0*v(i,j,k)
     $                    +.25e0*v(i+1,j,k))
           endif
           uf(i,j,k)=u3bc(n,k)
          endif

          if(ktype.eq.8) then
           if(u(i+1,j,k).ge.0.) then
            uf(i+1,j,k)=u3bc(n,k)
           else
            uf(i+1,j,k)=ga*(.25e0*u(i+2,j-1,k)+.5e0*u(i+2,j,k)
     $                    +.25e0*u(i+2,j+1,k))
     $                 +(1.e0-ga)*(.25e0*u(i+1,j-1,k)+.5e0*u(i+1,j,k)
     $                   +.25e0*u(i+1,j+1,k))
           endif
           uf(i,j,k)=uf(i+1,j,k)
           if(v(i,j,k).le.0.) then
            vf(i,j,k)=v3bc(n,k)
           else
            vf(i,j,k)=ga*(.25e0*v(i-1,j-1,k)+.5e0*v(i,j-1,k)
     $                     +.25e0*v(i+1,j-1,k))
     $                  +(1.e0-ga)*(.25e0*v(i-1,j,k)+.5e0*v(i,j,k)
     $                    +.25e0*v(i+1,j,k))
           endif
          endif
          
         enddo
        enddo
       endif

        do k=1,kbm1
          do j=1,jm
            do i=1,im
              uf(i,j,k)=uf(i,j,k)*dum(i,j)
              vf(i,j,k)=vf(i,j,k)*dvm(i,j)
            end do
          end do
        end do
C
        return
C
      else if(idx.eq.4) then

C-----------------------------------------------------------------------
C
C     Temperature and salinity boundary conditions (using uf and vf,
C     respectively):
C
       if(nibc.gt.0) then
        do n=1,nibc
         i=ibc(1,n)
         j=ibc(2,n)
         ktype=ibc(3,n)
         do k=1,kbm1
          
          if(ktype.eq.0) then
           uf(i,j,k)=t3bc(n,k)
          endif
          
          if(ktype.eq.1) then
            u1=u(i+1,j,k)*dti/ds
            if(u1.ge.0.e0) then
              uf(i,j,k)=t3bc(n,k)-u1*(t(i+1,j,k)-t3bc(n,k))
c*              uf(i,j,k)=t(i,j,k)-u1*(t(i,j,k)-t3bc(n,k))
            else
              uf(i,j,k)=t(i,j,k)-u1*(t(i+1,j,k)-t(i,j,k))
              if(k.ne.1.and.k.ne.kbm1) then
                wm=.5e0*(w(i+1,j,k)+w(i+1,j,k+1))*dti
     $              /((zz(k-1)-zz(k+1))*dt(i+1,j))
                uf(i,j,k)=uf(i,j,k)-wm*(t(i+1,j,k-1)-t(i+1,j,k+1))
              endif
            endif
          endif

c calculate total flux from both open sides
          if(ktype.eq.2) then
            u1=u(i+1,j,k)*dti/ds
            if(u1.ge.0.e0) then
              uf1=t3bc(n,k)-u1*(t(i+1,j,k)-t3bc(n,k))
            else
              uf1=t(i,j,k)-u1*(t(i+1,j,k)-t(i,j,k))
              if(k.ne.1.and.k.ne.kbm1) then
                wm=.5e0*(w(i+1,j,k)+w(i+1,j,k+1))*dti
     $              /((zz(k-1)-zz(k+1))*dt(i+1,j))
                uf1=uf1-wm*(t(i+1,j,k-1)-t(i+1,j,k+1))
              endif
            endif
            u1=v(i,j+1,k)*dti/ds
            if(u1.ge.0.e0) then
              uf2=t3bc(n,k)-u1*(t(i,j+1,k)-t3bc(n,k))
            else
              uf2=t(i,j,k)-u1*(t(i,j+1,k)-t(i,j,k))
              if(k.ne.1.and.k.ne.kbm1) then
                wm=.5e0*(w(i,j+1,k)+w(i,j+1,k+1))*dti
     $              /((zz(k-1)-zz(k+1))*dt(i,j+1))
                uf2=uf2-wm*(t(i,j+1,k-1)-t(i,j+1,k+1))
              endif
            endif
c            uf(i,j,k)=uf1+uf2
            uf(i,j,k)=0.5*(uf1+uf2)
          endif

          if(ktype.eq.3) then
            u1=v(i,j+1,k)*dti/ds
            if(u1.ge.0.e0) then
              uf(i,j,k)=t3bc(n,k)-u1*(t(i,j+1,k)-t3bc(n,k))
c*              uf(i,j,k)=t(i,j,k)-u1*(t(i,j,k)-t3bc(n,k))
            else
              uf(i,j,k)=t(i,j,k)-u1*(t(i,j+1,k)-t(i,j,k))
              if(k.ne.1.and.k.ne.kbm1) then
                wm=.5e0*(w(i,j+1,k)+w(i,j+1,k+1))*dti
     $              /((zz(k-1)-zz(k+1))*dt(i,j+1))
                uf(i,j,k)=uf(i,j,k)-wm*(t(i,j+1,k-1)-t(i,j+1,k+1))
              endif
            endif
          endif

c calculate total flux from both open sides
          if(ktype.eq.4) then
            u1=u(i,j,k)*dti/ds
            if(u1.le.0.e0) then
              uf1=t3bc(n,k)-u1*(t3bc(n,k)-t(i-1,j,k))
            else
              uf1=t(i,j,k)-u1*(t(i,j,k)-t(i-1,j,k))
              if(k.ne.1.and.k.ne.kbm1) then
                wm=.5e0*(w(i-1,j,k)+w(i-1,j,k+1))*dti
     $              /((zz(k-1)-zz(k+1))*dt(i-1,j))
                uf1=uf1-wm*(t(i-1,j,k-1)-t(i-1,j,k+1))
              endif
            endif
            u1=v(i,j+1,k)*dti/ds
            if(u1.ge.0.e0) then
              uf2=t3bc(n,k)-u1*(t(i,j+1,k)-t3bc(n,k))
            else
              uf2=t(i,j,k)-u1*(t(i,j+1,k)-t(i,j,k))
              if(k.ne.1.and.k.ne.kbm1) then
                wm=.5e0*(w(i,j+1,k)+w(i,j+1,k+1))*dti
     $              /((zz(k-1)-zz(k+1))*dt(i,j+1))
                uf2=uf2-wm*(t(i,j+1,k-1)-t(i,j+1,k+1))
              endif
            endif
c            uf(i,j,k)=uf1+uf2
            uf(i,j,k)=0.5*(uf1+uf2)
          endif

          if(ktype.eq.5) then
            u1=u(i,j,k)*dti/ds
            if(u1.le.0.e0) then
              uf(i,j,k)=t3bc(n,k)-u1*(t3bc(n,k)-t(i-1,j,k))
c*              uf(i,j,k)=t(i,j,k)-u1*(t3bc(n,k)-t(i,j,k))
            else
              uf(i,j,k)=t(i,j,k)-u1*(t(i,j,k)-t(i-1,j,k))
              if(k.ne.1.and.k.ne.kbm1) then
                wm=.5e0*(w(i-1,j,k)+w(i-1,j,k+1))*dti
     $              /((zz(k-1)-zz(k+1))*dt(i-1,j))
                uf(i,j,k)=uf(i,j,k)-wm*(t(i-1,j,k-1)-t(i-1,j,k+1))
              endif
            endif
          endif

c calculate total flux from both open sides
          if(ktype.eq.6) then
            u1=u(i,j,k)*dti/ds
            if(u1.le.0.e0) then
              uf1=t3bc(n,k)-u1*(t3bc(n,k)-t(i-1,j,k))
            else
              uf1=t(i,j,k)-u1*(t(i,j,k)-t(i-1,j,k))
              if(k.ne.1.and.k.ne.kbm1) then
                wm=.5e0*(w(i-1,j,k)+w(i-1,j,k+1))*dti
     $              /((zz(k-1)-zz(k+1))*dt(i-1,j))
                uf1=uf1-wm*(t(i-1,j,k-1)-t(i-1,j,k+1))
              endif
            endif
            u1=v(i,j,k)*dti/ds
            if(u1.le.0.e0) then
              uf2=t3bc(n,k)-u1*(t3bc(n,k)-t(i,j-1,k))
            else
              uf2=t(i,j,k)-u1*(t(i,j,k)-t(i,j-1,k))
              if(k.ne.1.and.k.ne.kbm1) then
                wm=.5e0*(w(i,j-1,k)+w(i,j-1,k+1))*dti
     $              /((zz(k-1)-zz(k+1))*dt(i,j-1))
                uf2=uf2-wm*(t(i,j-1,k-1)-t(i,j-1,k+1))
              endif
            endif
c            uf(i,j,k)=uf1+uf2
            uf(i,j,k)=0.5*(uf1+uf2)
          endif

          if(ktype.eq.7) then
            u1=v(i,j,k)*dti/ds
            if(u1.le.0.e0) then
              uf(i,j,k)=t3bc(n,k)-u1*(t3bc(n,k)-t(i,j-1,k))
c*              uf(i,j,k)=t(i,j,k)-u1*(t3bc(n,k)-t(i,j,k))
            else
              uf(i,j,k)=t(i,j,k)-u1*(t(i,j,k)-t(i,j-1,k))
              if(k.ne.1.and.k.ne.kbm1) then
                wm=.5e0*(w(i,j-1,k)+w(i,j-1,k+1))*dti
     $              /((zz(k-1)-zz(k+1))*dt(i,j-1))
                uf(i,j,k)=uf(i,j,k)-wm*(t(i,j-1,k-1)-t(i,j-1,k+1))
              endif
            endif
          endif

c calculate total flux from both open sides
          if(ktype.eq.8) then
            u1=u(i+1,j,k)*dti/ds
            if(u1.ge.0.e0) then
              uf1=t3bc(n,k)-u1*(t(i+1,j,k)-t3bc(n,k))
            else
              uf1=t(i,j,k)-u1*(t(i+1,j,k)-t(i,j,k))
              if(k.ne.1.and.k.ne.kbm1) then
                wm=.5e0*(w(i+1,j,k)+w(i+1,j,k+1))*dti
     $              /((zz(k-1)-zz(k+1))*dt(i+1,j))
                uf1=uf1-wm*(t(i+1,j,k-1)-t(i+1,j,k+1))
              endif
            endif
            u1=v(i,j,k)*dti/ds
            if(u1.le.0.e0) then
              uf2=t3bc(n,k)-u1*(t3bc(n,k)-t(i,j-1,k))
            else
              uf2=t(i,j,k)-u1*(t(i,j,k)-t(i,j-1,k))
              if(k.ne.1.and.k.ne.kbm1) then
                wm=.5e0*(w(i,j-1,k)+w(i,j-1,k+1))*dti
     $              /((zz(k-1)-zz(k+1))*dt(i,j-1))
                uf2=uf2-wm*(t(i,j-1,k-1)-t(i,j-1,k+1))
              endif
            endif
c            uf(i,j,k)=uf1+uf2
            uf(i,j,k)=0.5*(uf1+uf2)
          endif

         enddo
        enddo
       endif

        do k=1,kbm1
          do j=1,jm
            do i=1,im
              uf(i,j,k)=uf(i,j,k)*fsm(i,j)
              vf(i,j,k)=vf(i,j,k)*fsm(i,j)
            end do
          end do
        end do
C
        return
C
      else if(idx.eq.5) then
C
C     Vertical velocity boundary conditions:
C
        do k=1,kbm1
          do j=1,jm
            do i=1,im
              w(i,j,k)=w(i,j,k)*fsm(i,j)
            end do
          end do
        end do
C
        return
C
      else if(idx.eq.6) then
C
C     q2 and q2l boundary conditions:
C
       if(nibc.gt.0) then
        do n=1,nibc
         i=ibc(1,n)
         j=ibc(2,n)
         ktype=ibc(3,n)

         do k=2,kbm1

          if(ktype.eq.0) then
           uf(i,j,k)=umol
           vf(i,j,k)=umol
          endif
          
          if(ktype.eq.1) then
            u1=0.5*(u(i+1,j,k)+u(i+1,j,k-1))*dti/ds
            if(u1.ge.0.e0) then
              uf(i,j,k)=umol-u1*(q2(i+1,j,k)-umol)
              vf(i,j,k)=umol-u1*(q2l(i+1,j,k)-umol)
c*              uf(i,j,k)=q2(i,j,k)-u1*(q2(i,j,k)-umol)
c*              vf(i,j,k)=q2l(i,j,k)-u1*(q2l(i,j,k)-umol)
            else
              uf(i,j,k)=q2(i,j,k)-u1*(q2(i+1,j,k)-q2(i,j,k))
              vf(i,j,k)=q2l(i,j,k)-u1*(q2l(i+1,j,k)-q2l(i,j,k))
            endif
          endif

c calculate total flux from both open sides
          if(ktype.eq.2) then
            u1=u(i+1,j,k)*dti/ds
            if(u1.ge.0.e0) then
              uf1=umol-u1*(q2(i+1,j,k)-umol)
              vf1=umol-u1*(q2l(i+1,j,k)-umol)
            else
              uf1=q2(i,j,k)-u1*(q2(i+1,j,k)-q2(i,j,k))
              vf1=q2l(i,j,k)-u1*(q2l(i+1,j,k)-q2l(i,j,k))
              if(k.ne.1.and.k.ne.kbm1) then
                wm=.5e0*(w(i+1,j,k)+w(i+1,j,k+1))*dti
     $              /((zz(k-1)-zz(k+1))*dt(i+1,j))
                uf1=uf1-wm*(q2(i+1,j,k-1)-q2(i+1,j,k+1))
                vf1=vf1-wm*(q2l(i+1,j,k-1)-q2l(i+1,j,k+1))
              endif
            endif
            u1=v(i,j+1,k)*dti/ds
            if(u1.ge.0.e0) then
              uf2=t3bc(n,k)-u1*(t(i,j+1,k)-t3bc(n,k))
            else
              uf2=t(i,j,k)-u1*(t(i,j+1,k)-t(i,j,k))
              if(k.ne.1.and.k.ne.kbm1) then
                wm=.5e0*(w(i,j+1,k)+w(i,j+1,k+1))*dti
     $              /((zz(k-1)-zz(k+1))*dt(i,j+1))
                uf2=uf2-wm*(t(i,j+1,k-1)-t(i,j+1,k+1))
              endif
            endif
c            uf(i,j,k)=uf1+uf2
            uf(i,j,k)=0.5*(uf1+uf2)
          endif

          if(ktype.eq.3) then
            u1=0.5*(v(i,j+1,k)+v(i,j+1,k-1))*dti/ds
            if(u1.ge.0.e0) then
              uf(i,j,k)=umol-u1*(q2(i,j+1,k)-umol)
              vf(i,j,k)=umol-u1*(q2l(i,j+1,k)-umol)
c*              uf(i,j,k)=q2(i,j,k)-u1*(q2(i,j,k)-umol)
c*              vf(i,j,k)=q2l(i,j,k)-u1*(q2l(i,j,k)-umol)
            else
              uf(i,j,k)=q2(i,j,k)-u1*(q2(i,j+1,k)-q2(i,j,k))
              vf(i,j,k)=q2l(i,j,k)-u1*(q2l(i,j+1,k)-q2l(i,j,k))
            endif
          endif

c calculate total flux from both open sides
          if(ktype.eq.4) then
            u1=u(i,j,k)*dti/ds
            if(u1.le.0.e0) then
              uf1=t3bc(n,k)-u1*(t3bc(n,k)-t(i-1,j,k))
            else
              uf1=t(i,j,k)-u1*(t(i,j,k)-t(i-1,j,k))
              if(k.ne.1.and.k.ne.kbm1) then
                wm=.5e0*(w(i-1,j,k)+w(i-1,j,k+1))*dti
     $              /((zz(k-1)-zz(k+1))*dt(i-1,j))
                uf1=uf1-wm*(t(i-1,j,k-1)-t(i-1,j,k+1))
              endif
            endif
            u1=v(i,j+1,k)*dti/ds
            if(u1.ge.0.e0) then
              uf2=t3bc(n,k)-u1*(t(i,j+1,k)-t3bc(n,k))
            else
              uf2=t(i,j,k)-u1*(t(i,j+1,k)-t(i,j,k))
              if(k.ne.1.and.k.ne.kbm1) then
                wm=.5e0*(w(i,j+1,k)+w(i,j+1,k+1))*dti
     $              /((zz(k-1)-zz(k+1))*dt(i,j+1))
                uf2=uf2-wm*(t(i,j+1,k-1)-t(i,j+1,k+1))
              endif
            endif
c            uf(i,j,k)=uf1+uf2
            uf(i,j,k)=0.5*(uf1+uf2)
          endif

          if(ktype.eq.5) then
            u1=0.5*(u(i,j,k)+u(i,j,k-1))*dti/ds
            if(u1.le.0.e0) then
              uf(i,j,k)=umol-u1*(umol-q2(i-1,j,k))
              vf(i,j,k)=umol-u1*(umol-q2l(i-1,j,k))
c*              uf(i,j,k)=q2(i,j,k)-u1*(umol-q2(i,j,k))
c*              vf(i,j,k)=q2l(i,j,k)-u1*(umol-q2l(i,j,k))
            else
              uf(i,j,k)=q2(i,j,k)-u1*(q2(i,j,k)-q2(i-1,j,k))
              vf(i,j,k)=q2l(i,j,k)-u1*(q2l(i,j,k)-q2l(i-1,j,k))
            endif
          endif

c calculate total flux from both open sides
          if(ktype.eq.6) then
            u1=u(i,j,k)*dti/ds
            if(u1.le.0.e0) then
              uf1=t3bc(n,k)-u1*(t3bc(n,k)-t(i-1,j,k))
            else
              uf1=t(i,j,k)-u1*(t(i,j,k)-t(i-1,j,k))
              if(k.ne.1.and.k.ne.kbm1) then
                wm=.5e0*(w(i-1,j,k)+w(i-1,j,k+1))*dti
     $              /((zz(k-1)-zz(k+1))*dt(i-1,j))
                uf1=uf1-wm*(t(i-1,j,k-1)-t(i-1,j,k+1))
              endif
            endif
            u1=v(i,j,k)*dti/ds
            if(u1.le.0.e0) then
              uf2=t3bc(n,k)-u1*(t3bc(n,k)-t(i,j-1,k))
            else
              uf2=t(i,j,k)-u1*(t(i,j,k)-t(i,j-1,k))
              if(k.ne.1.and.k.ne.kbm1) then
                wm=.5e0*(w(i,j-1,k)+w(i,j-1,k+1))*dti
     $              /((zz(k-1)-zz(k+1))*dt(i,j-1))
                uf2=uf2-wm*(t(i,j-1,k-1)-t(i,j-1,k+1))
              endif
            endif
c            uf(i,j,k)=uf1+uf2
            uf(i,j,k)=0.5*(uf1+uf2)
          endif

          if(ktype.eq.7) then
            u1=0.5*(v(i,j,k)+v(i,j,k))*dti/ds
            if(u1.le.0.e0) then
              uf(i,j,k)=umol-u1*(umol-q2(i,j-1,k))
              vf(i,j,k)=umol-u1*(umol-q2l(i,j-1,k))
c*              uf(i,j,k)=q2(i,j,k)-u1*(umol-q2(i,j,k))
c*              vf(i,j,k)=q2l(i,j,k)-u1*(umol-q2l(i,j,k))
            else
              uf(i,j,k)=q2(i,j,k)-u1*(q2(i,j,k)-q2(i,j-1,k))
              vf(i,j,k)=q2l(i,j,k)-u1*(q2l(i,j,k)-q2l(i,j-1,k))
            endif
          endif

c calculate total flux from both open sides
          if(ktype.eq.8) then
            u1=u(i+1,j,k)*dti/ds
            if(u1.ge.0.e0) then
              uf1=t3bc(n,k)-u1*(t(i+1,j,k)-t3bc(n,k))
            else
              uf1=t(i,j,k)-u1*(t(i+1,j,k)-t(i,j,k))
              if(k.ne.1.and.k.ne.kbm1) then
                wm=.5e0*(w(i+1,j,k)+w(i+1,j,k+1))*dti
     $              /((zz(k-1)-zz(k+1))*dt(i+1,j))
                uf1=uf1-wm*(t(i+1,j,k-1)-t(i+1,j,k+1))
              endif
            endif
            u1=v(i,j,k)*dti/ds
            if(u1.le.0.e0) then
              uf2=t3bc(n,k)-u1*(t3bc(n,k)-t(i,j-1,k))
            else
              uf2=t(i,j,k)-u1*(t(i,j,k)-t(i,j-1,k))
              if(k.ne.1.and.k.ne.kbm1) then
                wm=.5e0*(w(i,j-1,k)+w(i,j-1,k+1))*dti
     $              /((zz(k-1)-zz(k+1))*dt(i,j-1))
                uf2=uf2-wm*(t(i,j-1,k-1)-t(i,j-1,k+1))
              endif
            endif
c            uf(i,j,k)=uf1+uf2
            uf(i,j,k)=0.5*(uf1+uf2)
          endif
c*
         enddo
        end do
       endif

C
        do k=1,kb
          do j=1,jm
            do i=1,im
              uf(i,j,k)=uf(i,j,k)*fsm(i,j)+1.e-10
              vf(i,j,k)=vf(i,j,k)*fsm(i,j)+1.e-10
            end do
          end do
        end do
C
        return
C
      endif
C
      ELSE
C-----------------------------------------------------------------------
C  CLOSED BOUNDARY CASE
C-----------------------------------------------------------------------
      if(idx.eq.1) then

C     Elevation 

        do j=1,jm
          do i=1,im
            elf(i,j)=elf(i,j)*fsm(i,j)
          end do
        end do

        return

      else if(idx.eq.2) then
C
C     External (2-D) velocity:
C
        do j=1,jm
          do i=1,im
            uaf(i,j)=uaf(i,j)*dum(i,j)
            vaf(i,j)=vaf(i,j)*dvm(i,j)
          end do
        end do
C
        return
C
      else if(idx.eq.3) then
C
C-----------------------------------------------------------------------
C
C     Internal (3-D) boundary conditions:
C
C     Velocity 
C
        do k=1,kbm1
          do j=1,jm
            do i=1,im
              uf(i,j,k)=uf(i,j,k)*dum(i,j)
              vf(i,j,k)=vf(i,j,k)*dvm(i,j)
            end do
          end do
        end do
C
        return
C
      else if(idx.eq.4) then
C
C     Temperature and salinity boundary conditions (using uf and vf,
C     respectively):
C
        do k=1,kbm1
          do j=1,jm
            do i=1,im
              uf(i,j,k)=uf(i,j,k)*fsm(i,j)
              vf(i,j,k)=vf(i,j,k)*fsm(i,j)
            end do
          end do
        end do
C
        return
C
      else if(idx.eq.5) then
C
C     Vertical velocity boundary conditions:
C
        do k=1,kbm1
          do j=1,jm
            do i=1,im
              w(i,j,k)=w(i,j,k)*fsm(i,j)
            end do
          end do
        end do
C
        return
C
      else if(idx.eq.6) then
C
C     q2 and q2l boundary conditions:
C
        do k=1,kb
          do j=1,jm
            do i=1,im
              uf(i,j,k)=uf(i,j,k)*fsm(i,j)+1.e-10
              vf(i,j,k)=vf(i,j,k)*fsm(i,j)+1.e-10
            end do
          end do
        end do
C
        return
C
      endif
C
      endif
      
      end
C
